#! /bin/csh -f

# procspec2.csh : Procedure for the first-step processing of OHS/CISCO

if ( $#argv != 3 ) then
    echo "Usage : "$0" MODE(OHS/CISCO) Y1 Y2"
    exit 1
endif

set OHSMODE=$1
set Y1=$2
set Y2=$3

set CLCOM=/tmp/procspec.cl.com
set TMPAWK=/tmp/procspec.tmp.awk
set TMPVAL=/tmp/procspec.tmpvl
set TMPVAL1=/tmp/procspec.tmpvl1
set TMPVAL2=/tmp/procspec.tmpvl2

#create awk script
echo '{print($2-$5)}' > $TMPAWK

# Combine to create object and sky frames
cat << EOF > $CLCOM
 imdel test1,test2,test3,ftest1,testp,testn,test1p
 imcombine @obj.tbl testp reject=sigclip 
 imcombine @sky.tbl testn reject=sigclip 
EOF

# Subtract sky frame
cat << EOF >> $CLCOM
 imarith testp - testn test1 
EOF

# Remove offset in quad 1 ([1:512,513:1024])
cat << EOF >> $CLCOM
 del $TMPVAL1,$TMPVAL2
 imstatrej test1[413:512,463:512] 2 > $TMPVAL1
 imstatrej test1[413:512,513:562] 2 > $TMPVAL2
 !paste  $TMPVAL1 $TMPVAL2 | gawk -f $TMPAWK > $TMPVAL 
 imarith test1[1:512,513:1024] + @${TMPVAL} test3 
 imcopy test3 test1[1:512,513:1024] 
EOF

# Remove excess dark current
if ( ${OHSMODE} == "OHS" ) then
cat << EOF >> $CLCOM
 imdel test3,test31,test32
 imcopy test1[*,681:770] test31
 !rejbadpix3.csh test31 3 21 5 5 3
 blkavg test31 test31 1 1024 
 imcopy test1[*,251:380] test32
 !rejbadpix3.csh test32 3 21 5 5 3
 blkavg test32 test32 1 1024 
 imcombine test31,test32 test3 combine=average
 boxcar test3 test3 9 1 boundar=nearest
 magnify test3 test3 1 .0009765625 y2=1024 dy=1 interpo=nearest boundar=nearest fluxcon-
 imarith test1 - test3 test1
EOF
endif

# Reject bad pixels by median filtering.
#imcopy test1[*,421:660] test1p
set Y512=`gawk -v y=$Y1 'BEGIN{print(513-y)}'`
set Y513=`gawk -v y=$Y1 'BEGIN{print(513-y+1)}'`
set DY=`gawk -v y1=$Y1 -v y2=$Y2 'BEGIN{print(y2-y1+1)}'`
cat << EOF >> $CLCOM
 imcopy test1[*,${Y1}:${Y2}] test1p
# !rejbadpix3.csh test1p 1 20 6 6 3
# !rejbadpix3.csh test1p 1 20 5 5 1
# !rejbadpix3.csh test1p 1 20 4 4 1

 imdel tmprej1.fits
 imcopy test1p[21:400,1:${Y512}] tmprej1.fits
 !foddeven tmprej1.fits tmprej1.fits
 imcopy tmprej1.fits test1p[21:400,1:${Y512}]
 imdel tmprej1.fits
 imcopy test1p[543:962,1:${Y512}] tmprej1.fits
 !foddeven tmprej1.fits tmprej1.fits
 imcopy tmprej1.fits test1p[543:962,1:${Y512}]


 imdel tmprej1.fits
 imcopy test1p[21:400,${Y513}:${DY}] tmprej1.fits
 !foddeven tmprej1.fits tmprej1.fits
 imcopy tmprej1.fits test1p[21:400,${Y513}:${DY}]
 imdel tmprej1.fits
 imcopy test1p[543:962,${Y513}:${DY}] tmprej1.fits
 !foddeven tmprej1.fits tmprej1.fits
 imcopy tmprej1.fits test1p[543:962,${Y513}:${DY}]

# !rejbadpix3.csh test1p 1 20 4 4 1
 
 #imcopy test1p test1[*,421:660]
 imcopy test1p test1[*,${Y1}:${Y2}]
EOF

# Divide by flat frame 
cat << EOF >> $CLCOM
 imarith test1 / flat.fits test1
EOF

# Correct the distortion of slit image
# (only for CISCO data)

if ( ${OHSMODE} == "OHS" ) then
cat << EOF >> $CLCOM
# !frbline test1 ftest1 
 imcopy test1 ftest1.fits
EOF
else
cat << EOF >> $CLCOM
 imdel ftest1.fits
 imcopy test1 ftest1.fits
 !frbline ftest1.fits ftest1.fits 
EOF
endif

# Cut-out the portion of the spectrum, and remove residual sky emission.
#imcopy ftest1[*,421:660] test2 
cat << EOF >> $CLCOM
 imcopy ftest1[*,${Y1}:${Y2}] test2 
 fit1d test2 test2 axis=2 type=difference interac- functio=legendre order=1 low_rej=4 high_rej=4 niterat=3 grow=2 naverage=1
 !rejbadpix3.csh test2 1 20 6 6 3
 !rejbadpix3.csh test2 1 20 5 5 1
 !rejbadpix3.csh test2 1 20 4 4 1
 !rejbadpix3.csh test2 1 20 4 4 1
 disp test2 1 fi+ 
 logout
EOF

cat $CLCOM | cl
