#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "fitsio.h"
#include "nrutil.h"

#define LINEPIX 1024
#define LINESCAN 1024
#define MAXDATA 1024
#define MAXCHAR 1024
#define MAXHDR 1024
#define HDRSIZE 80

main(int argc, char **argv)
{
  fitsfile *fp;
  fitsfile *fpfits;
  FILE *fpscan;
  int i,j;
  int flag;
  int nscan;
  int istart,iend;
  int masktype;

  double a,b,c;
  float **mat,**invmat;
  float *col,*vec;
  float d;
  int *indx;
  int nmatrix;
  
  int base_x, intdy, x1, x2;
  double base_y,dy,ddy;

  int status,nfound,anynull,bitpix=-32;
  long fpixel, nelements,naxis=2,npixels;
  long naxes[2]={LINEPIX, LINEPIX},npix;
  float nullval; 
  char line[MAXCHAR];
  float *pix,*basepix;

  float *x, *y, *fsigma, *fpeak;
  float peak,sigma,peakmax,pix1,pix2;

  char openfile[100],openfits[100],savefile[100];

  int nkeys;
  char *hdrptr[MAXHDR];

  if(argc!=5){
    fprintf(stderr,"Usage : %s scan_file 1D_skyflat_file measured_x output_file \n",argv[0]);
    exit(1);
  }

  sprintf(openfile, "%s",argv[1]);
  sprintf(openfits, "%s",argv[2]);
  base_x=atoi(argv[3]);
  sprintf(savefile, "%s",argv[4]);

  /* allocate memory */
  x=(float *) malloc(LINESCAN*sizeof(float));
  y=(float *) malloc(LINESCAN*sizeof(float));
  fsigma=(float *) malloc(LINESCAN*sizeof(float));
  fpeak=(float *) malloc(LINESCAN*sizeof(float));

  /* allocate matrix */
  nmatrix=3;
  mat=matrix(1,nmatrix,1,nmatrix);
  invmat=matrix(1,nmatrix,1,nmatrix);
  indx=ivector(1,nmatrix);
  col=vector(1,nmatrix);
  vec=vector(1,nmatrix);

  /* load scan file */
  fprintf(stderr,"%s \t: Read %s  ... ",argv[0],openfile);
  if((fpscan=fopen(openfile,"r"))==NULL){
    fprintf(stderr,"Open Error  %s",openfile);
    exit(1);
  }
  i=0;
  nscan=0;

  fgets(line,MAXCHAR,fpscan); /* dummy read*/

  while(fgets(line,MAXCHAR,fpscan)!=NULL){
    sscanf(line,"%f %f %f %f",x,y,fsigma,fpeak);
    x++;
    y++;
    fsigma++;
    fpeak++;
    nscan++;
  }
  x-=nscan;
  y-=nscan;
  fsigma-=nscan;
  fpeak-=nscan;
  fclose(fpscan);
  fprintf(stderr,"OK\n");

  /* load template skyflat frame */
  fprintf(stderr,"%s \t: Read %s  ... \n",argv[0],openfits);
  status=0;
  if(fits_open_file(&fpfits,openfits, READONLY, &status)){
    fits_report_error(stderr,status);
    return(status);
  }

  if(fits_read_keys_lng(fpfits, "NAXIS", 1, 2, naxes, &nfound, &status) || fits_read_keys_lng(fpfits, "NAXIS", 1, 1, naxes, &nfound, &status)){
    fits_report_error(stderr,status);
    return(status);
  }

  if((naxes[0]!=1)&&(naxes[1]!=1)){
    fprintf(stderr,"1-D image only.\n");
    exit(1);
  }
  
  if(fits_get_hdrspace(fpfits, &nkeys, NULL, &status)){
    fits_report_error(stderr,status);
    return(status);
  }
  for (i=0;i<nkeys;i++){
    hdrptr[i]=malloc(HDRSIZE);
    if(fits_read_record(fpfits,i+1,hdrptr[i],&status)){
      fits_report_error(stderr,status);
      return(status);
    }
  }

  npix=naxes[0]*naxes[1];
  basepix=(float *) malloc(npix*sizeof(float));

  fpixel = 1;
  nullval = 0;
  fprintf(stderr,"naxes0=%d naxes1=%d\n",naxes[0],naxes[1]);
  if (fits_read_img(fpfits, TFLOAT, fpixel, npix, &nullval, basepix, &anynull, &status)){
    fits_report_error(stderr,status);
    return(status);
  }

  if(fits_close_file(fpfits, &status)){
    fits_report_error(stderr,status);
    return(status);
  }

  fprintf(stderr,"OK.\n");

  /* find pixel range to use */
  peakmax=fpeak[0];
  for(i=0;i<nscan;i++) if(peakmax<fpeak[i]) peakmax=fpeak[i];
  fprintf(stderr,"peak %f\n",peakmax);
  istart=-1;iend=-1;
  for(i=0;i<nscan;i++){
    if(istart==-1 && fpeak[i]>(peakmax/5)) istart=i;
    if(iend==-1 && istart!=-1 && x[i]>800 && fpeak[i]<(peakmax/5)) iend=i;
  }
  if(istart==-1) istart=0;
  if(iend==-1) iend=nscan;
  fprintf(stderr,"%f - %f\n",x[istart],x[iend-1]);

  /* final fit */
  /* calculate matrix */
  for(j=1;j<=nmatrix;j++){
    vec[j]=0.0;
    for(i=1;i<=nmatrix;i++) mat[i][j]=0.0;
  }
  for(i=istart;i<iend;i++){
    if(fpeak[i]>(peakmax/10.0) && fpeak[i]>100.0){
      mat[1][1]+=pow(x[i],4);
      mat[1][2]+=pow(x[i],3);
      mat[2][1]+=pow(x[i],3);
      mat[1][3]+=pow(x[i],2);
      mat[2][2]+=pow(x[i],2);
      mat[3][1]+=pow(x[i],2);
      mat[3][2]+=x[i];
      mat[2][3]+=x[i];
      mat[3][3]+=1.0;
      
      vec[1]+=x[i]*x[i]*y[i];
      vec[2]+=x[i]*y[i];
      vec[3]+=y[i];
    }
  }
  
  /* calculate inverse */
  ludcmp(mat,nmatrix,indx,&d);
  for(j=1;j<=nmatrix;j++){
    for(i=1;i<=nmatrix;i++) col[i]=0.0;
    col[j]=1.0;
    lubksb(mat,nmatrix,indx,col);
    for(i=1;i<=nmatrix;i++) invmat[i][j]=col[i];
  }
  
  /* caluculate fit parameter */
  a=0.0;b=0.0;c=0.0;
  for(i=1;i<=3;i++){
    a+=(double) (invmat[i][1]*vec[i]);
    b+=(double) (invmat[i][2]*vec[i]);
    c+=(double) (invmat[i][3]*vec[i]);
  }
  
  fprintf(stderr,"%e %e %e\n",a,b,c);

  /* free matrix */
  free_matrix(mat,1,nmatrix,1,nmatrix);
  free_matrix(invmat,1,nmatrix,1,nmatrix);
  free_vector(col,1,nmatrix);
  free_vector(vec,1,nmatrix);
  free_ivector(indx,1,nmatrix);
  
  /* free memory */
  free(x);
  free(y);
  free(fsigma);
  free(fpeak);
  
  /* create skyflat frame */
  naxes[0]=1024;
  fprintf(stderr,"naxes1=%d 2=%d\n",naxes[0],naxes[1]);
  npixels=naxes[0]*naxes[1];
  pix=(float *) malloc(npixels*sizeof(float));

  base_y=a*base_x*base_x+b*base_x+c;
  fprintf(stderr,"basex=%d basey=%f\n",base_x, base_y);

  for(i=0;i<naxes[1];i++){
    for(j=0;j<naxes[0];j++){
      dy=base_y-(a*(j+1)*(j+1)+b*(j+1)+c);
      intdy=floor(dy);
      ddy=dy-intdy;

      if(i==1 && (j+1)==(base_x)) fprintf(stderr,"j=%d base_x=%d dy=%f intdy=%d ddy=%f\n",j,base_x,dy,intdy,ddy);

      x1=i+intdy;
      x2=i+intdy+1;
      
      if( 0 < x1 && x1 < naxes[1]) pix1=basepix[x1];
      else pix1=1;
      if( 0 < x2 && x2 < naxes[1]) pix2=basepix[x2];
      else pix2=1;
      
      pix[j+naxes[0]*i]=pix1*(1.0-ddy)+pix2*ddy;
    }
  }

  fprintf(stderr,"a=%e b=%e c=%e\n",a,b,c);
  
  /* create skyflat file */
  fprintf(stderr,"%s \t: Write %s  ... ",argv[0],savefile);
  unlink(savefile);

  status=0;
  if (fits_create_file(&fp, savefile, &status)){
    fits_report_error(stderr,status);
    return(status);
  }

  if (fits_create_img(fp, bitpix, naxis, naxes, &status)){
    fits_report_error(stderr,status);
    return(status);
  }
  
  if (fits_write_key(fp, TDOUBLE, "A", &a, "Y=a*X*X+b*X+c" , &status)){
    fits_report_error(stderr,status);
    return(status);
  }
  if (fits_write_key(fp, TDOUBLE, "B", &b, "Y=a*X*X+b*X+c" , &status)){
    fits_report_error(stderr,status);
    return(status);
  }
  if (fits_write_key(fp, TDOUBLE, "C", &c, "Y=a*X*X+b*X+c" , &status)){
    fits_report_error(stderr,status);
    return(status);
  }
  
  fpixel=1;
  if (fits_write_img(fp, TFLOAT, fpixel, npixels, pix, &status)) 
    fits_report_error(stderr,status);
    
  if (fits_close_file(fp, &status)){
    fits_report_error(stderr,status);
    return(status);
  }
  fprintf(stderr,"Done.\n");

  free(pix);
}
