#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "fitsio.h"

#define LINEPIX 1024
#define MAXCHAR 1024
#define MAXHDR 1024
#define HDRSIZE 80

main(int argc, char **argv)
{
  FILE *fp;
  fitsfile *fptr1,*fptr2;
  int status, nfound, anynull;
  long fpixel, npixels;
  int i,j;
  float nullval; 
  char openfile[100],savefile[100],paramfile[100];
  char grism[100];
  char filter02[64];
  float sumodd,sumeven,offsetodd,offseteven;

  int bitpix=-32;
  long naxis=2;

  long naxes[2]={LINEPIX, LINEPIX};
  static float buffer[LINEPIX],pix[LINEPIX][LINEPIX];
  long nbuffer=LINEPIX;

  float a,b,x0,xoffset;
  char line[MAXCHAR];
  char disperser[MAXCHAR];
 
  float y,subpixshift,residual;
  int pixshift;

  int nkeys;
  char *hdrptr[MAXHDR];

  int npixodd,npixeven;

  if(argc !=3){
    fprintf(stderr,"Usage : %s (input file) (output file)\n",argv[0]);
    exit(1);
  }

  sprintf(openfile, "%s",argv[1]);
  sprintf(savefile, "%s",argv[2]);
  
  printf("%s \t: Read %s  ... ",argv[0],openfile);

  status=0;

  if(fits_open_file(&fptr1,openfile, READONLY, &status)){
    fits_report_error(stderr,status);
    return(status);
  }

  /* Read Header Information */
  if(fits_read_keys_lng(fptr1, "NAXIS", 1, 2, naxes, &nfound, &status)){
    fits_report_error(stderr,status);
    return(status);
  }

  if((naxes[0]>LINEPIX)||(naxes[1]>LINEPIX)){
    fprintf(stderr,"Image is too large\n");
    exit(0);
  }
  fprintf(stderr,"%d x %d pixels\n",naxes[0],naxes[1]);
  
  if(fits_get_hdrspace(fptr1, &nkeys, NULL, &status)){
    fits_report_error(stderr,status);
    return(status);
  }
  for (i=0;i<nkeys;i++){
    hdrptr[i]=malloc(HDRSIZE);
    if(fits_read_record(fptr1,i+1,hdrptr[i],&status)){
      fits_report_error(stderr,status);
      return(status);
    }
  }

  /* naxes[0] :  x */
  /* naxes[1] : Կ y */
  /* j :  x*/
  /* i : Կ y*/
  /* pix[y][x] */

  sumodd=0;
  sumeven=0;
  nbuffer=naxes[0];
  fpixel = 1;
  nullval = 0;
  npixodd=0;
  npixeven=0;
  for(i=0;i<naxes[1];i++){
    if (fits_read_img(fptr1, TFLOAT, fpixel, nbuffer, &nullval,
		      buffer, &anynull, &status)){
      fits_report_error(stderr,status);
      return(status);
    }
    if(i%2==0) 
      for(j=0;j<naxes[0];j++){
	sumeven+=buffer[j];
	npixeven++;
      }
    else
      for(j=0;j<naxes[0];j++){
	sumodd+=buffer[j];
	npixodd++;
      }
    
    fpixel+=nbuffer;
  }
    
  sumodd/=((double) npixodd);
  sumeven/=((double) npixeven);

  offsetodd=(sumeven-sumodd)/2;
  offseteven=-offsetodd;

  fprintf(stderr,"sum odd=%lf even=%lf\n",sumodd,sumeven);
  fprintf(stderr,"off odd=%lf even=%lf\n",offsetodd,offseteven);

  fpixel = 1;
  nullval = 0;
  for(i=0;i<naxes[1];i++){
    if (fits_read_img(fptr1, TFLOAT, fpixel, nbuffer, &nullval,
		      buffer, &anynull, &status)){
      fits_report_error(stderr,status);
      return(status);
    }
    for(j=0;j<naxes[0];j++){
      if(i%2==0) pix[i][j]=buffer[j]+offseteven;
      else pix[i][j]=buffer[j]+offsetodd;
    }
    fpixel+=nbuffer;
  }

  if(fits_close_file(fptr1, &status)){
    fits_report_error(stderr,status);
    return(status);
  }

  printf("OK\n");

  printf("\t: Write %s  ... ",savefile);
  unlink(savefile);

  status=0;

  if (fits_create_file(&fptr2, savefile, &status)){
    fits_report_error(stderr,status);
    return(status);
  }

  /*  if (fits_create_img(fptr2, bitpix, naxis, naxes, &status)){
    fits_report_error(stderr,status);
    return(status);
  }
  */
  for (i=0;i<nkeys;i++){
    if(fits_write_record(fptr2,hdrptr[i],&status)){
      fits_report_error(stderr,status);
      return(status);
    }
  }

  fpixel=1;

  npixels=naxes[0];
  for(i=0;i<naxes[1];i++){
    if (fits_write_img(fptr2, TFLOAT, fpixel, npixels, pix[i], &status)) 
	  fits_report_error(stderr,status);
    fpixel+=npixels;
  }

  if (fits_close_file(fptr2, &status)){
    fits_report_error(stderr,status);
    return(status);
  }

  printf("OK\n");

}
